//////////////////////////////////////////////////////////////////////////
// TSXtruder 1.0 by Bendy's 3D Systems
// Copyright (c) 1999
//
// Based on Extruder 1.1 by Bryce Denney
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
//////////////////////////////////////////////////////////////////////////
//
// hfield.h
//
// The Hfield (height field) object does all the hard work of converting 
// the 2D image into a COB.  In the constructor (Hfield::Hfield), it creates 
// an array of hres*vres pixels and resamples the input image.  (The
// resampling is very poor, since it doesn't average pixel values 
// or antialias.)  After the constructor is called, Hfield::makecob()
// and its helper functions output the COB file.
//
//////////////////////////////////////////////////////////////////////////
//
// HField objects are created as polygon objects in TS, rather than written
// to file
//
//////////////////////////////////////////////////////////////////////////
#ifndef __HFIELD_H__
#define __HFIELD_H__

// these constants will be used to index the Hfield::bottom array
// Uneeded in the TSX
//const int NW=0,NE=1,SE=2,SW=3;
// I've defined my own. Bryce's consts are clockwise
// from below. I want clockwise from above.
#define	NE	0
#define	SE	1
#define	SW	2
#define	NW	3

const	int	F_TRIANGLE=0,F_PLANE=1,F_SEMI=2;

// Data types from tsx.h
struct CtsxFace;		
struct CtsxFaceVx;
struct tsxGNODE;
struct CtsxVector3f;

// Little define to set a face vertex
#define setFaceVert(my_face,vert_idx) { (my_face).vix=(my_face).uvix=vert_idx; }

const	PROGRESSBARINC=20;				
struct Hfield_pixel;     // declared later


class Hfield
{
	int hres, vres;       // horizontal and verticle grid resolution
	float base_size;      // size of that rectangular base that sits
	                      //   under the object.
	Hfield_pixel *data;   // resampled pixels. allocated in constructor.

	float	xsize,ysize;  // x and y size of final object
	char	*filename;	  // name of texture file
	int	add_texture,add_bumpmap;	// do we add textures?
	int face_type;					// do we triangulate top?

//	I don't need the bottom array or the file writing
//  functoins so they are removed
	// vertex numbers for the bottom 4 points
//	int bottom[4];
	// private functions called by makecob to do the dirty work.
//	void top_part(FILE *out);      // output file header
//	void worldvertices(FILE *out);
//	void texturevertices(FILE *out);
//	void faces(FILE *out);
//	void bottom_part(FILE *out);  // output the rest: material definition, etc.
	
	// Fuction to make a face from a face vertex list
	CtsxFace *makeFace(CtsxFace *face, CtsxFaceVx *verts, int n_verts);
	
public:
	Hfield(
		int _hres, int _vres,       // grid resolution (horiz and vert)
		float _base_size,           // height of the rectangular base
		float max_height,           // maximum height of extruded part
		TGAfile *tga,               // TGAfile object containing the 2D image
		float _x=1, float _y=1,		// x,y size of object
		int smooth=1,				// Smooth surface?
		double sfac=0.1,			// smoothing factor
		int face_t=F_PLANE,			// what type of faces?
		char *file=NULL,			// Name of the file. used for texture
		int texture=0,				// Apply texture
		int bump=0);				// Apply bump
		

//	int makecob(char *name);        // this function does all the rest.

	int insertObject(CTSXDialog &dlg);	// Inserts heightfield object into
										// Truespace 3
	int	calcNSemi(void);				// Calculate # of faces when semi-tri
private:
	float smoothPixel(TGAfile *tga, int x, int y, double param);
	void makeTriagAFaces(int vnum, CtsxVector3f *verts, int &i, CtsxFace **faces);
	void makeTriagBFaces(int vnum, CtsxVector3f *verts, int &i, CtsxFace **faces);
	void makePlaneFace(int vnum, CtsxVector3f *verts, int &i, CtsxFace **faces);
	void makeTriangularFaces(int vnum, CtsxVector3f *verts, int &i, CtsxFace **faces); 
	void makeSemiFaces(int vnum, CtsxVector3f *verts, int &i, CtsxFace **faces);
	BOOL isPlanar(void);
};

// The Hfield object will allocate one HField_pixel for each grid point.
// It stores the location of the grid point, and the vertex number of
// that point in the COB file.

struct Hfield_pixel
{
	float x,y,z;      // where is this vertex
	int vertex;       // vertex number for COB file
};
#endif
