//////////////////////////////////////////////////////////////////////////
// TSXtruder 1.0 by Bendy's 3D Systems
// Copyright (c) 1999
//
// Based on Extruder 1.1 by Bryce Denney
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
//////////////////////////////////////////////////////////////////////////
// Class to read in a 16/24 bit TGA file
#include <afx.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "tga.h"

TGA::TGA(short xi, short yi, RGB *i, char comp)
{

	id_length=0;
	c_map=0;
	img_type=comp|COL_TRUE;
	memset(c_map_spec,0,5);

	// setup image_spec pointers
	x_orig=(short *)(image_spec);
	y_orig=(short *)(image_spec+2);
	wid=(short *)(image_spec+4);
	hgt=(short *)(image_spec+6);
	depth=(image_spec+8);
	img_desc=(image_spec+9);

	*x_orig=0;
	*y_orig=yi;
	*wid=xi;
	*hgt=yi;
	*depth=24;
	*img_desc=ORIGIN_L | ORIGIN_T;
	
	image = i;
	grey=data=idf=NULL;
}

TGA::~TGA(void)
{
	deleteData();
}

int
TGA::read(char *filename)
{
	CFile ifp;
	int val;
		
	ifp.Open(filename,CFile::modeRead|CFile::typeBinary);
	if (!ifp) return TGA_ERR_UNK;
	val= read(ifp);
	ifp.Close();
	return val;
}

int
TGA::read(CFile &ifp)
{
	deleteData();
	ifp.Read(this,18);

	// currently only support truecolor uncompressed
	if (img_type!=COL_TRUE)
		return TGA_ERR_COLOUR;
	if ((img_type & RLE)==RLE)
		return TGA_ERR_COMP;


	if (*depth!=16 && *depth!=24)
		return TGA_ERR_COLDEPTH;


	// extract attribute info, no support for attribute as yet
	if (*img_desc & 0xf)
		return TGA_ERR_ATTRIB;

	// Image id field now if length in byte 0 > 0.
	if (id_length)
	{
		idf = (unsigned char *) new unsigned char[id_length];
		ifp.Read(idf,id_length);
	}

	// Read image info
	if (img_type!=0)
	{
		// read in uncompressed image
		data = new unsigned char[(*wid)*(*hgt)*(*depth)/8];
		readImage(ifp,data,*wid,*hgt,*depth);

		grey = new unsigned char[(*wid)*(*hgt)];
		// extract data to 8 bit greyscale.
		switch(*depth) {
			case(8):	memcpy(grey,data,(*wid)*(*hgt));break;
			case(16): {	unsigned char *p=data;	
						//truecolour 16 bit
						for(int i=0;i<(*wid)*(*hgt);i++,p+=2)
							grey[i]=grey16val((unsigned int *)p);

					break; }
			case(24): {	unsigned char *p=data;	
						//truecolour 24 bit
						for(int i=0;i<(*wid)*(*hgt);i++,p=p+3)
							grey[i]=grey24val(p);

					break; }
		}
	}
	return TGA_ERR_NONE;
}

void
TGA::close(void)
{
	deleteData();
}
char
TGA::grey16val(unsigned int *p)
{
	return (char)((255.0/31) * ((((*p)&BIT16_R)>>10) + (((*p)&BIT16_G)>>5) + ((*p)&BIT16_B))/3);
}
char
TGA::grey24val(unsigned char *p)
{
	return (*p + *(p+1) + *(p+2))/3;
}

void
readImage(CFile &ofp, unsigned char *image, int x, int y, int col_depth)
{
	ofp.Read(image,x*y*col_depth/8);
}

void
TGA::deleteData(void)
{
	if (grey!=NULL) { delete [] grey; grey=NULL;}
	if (image!=NULL) { delete [] image; image=NULL;}
	if (data!=NULL) { delete [] data; data=NULL;}
	if (idf!=NULL) { delete [] idf; idf=NULL;}
}
