//////////////////////////////////////////////////////////////////////////
// TSXtruder 1.0 by Bendy's 3D Systems
// Copyright (c) 1999
//
// Based on Extruder 1.1 by Bryce Denney
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
//////////////////////////////////////////////////////////////////////////// TGA specs taken from Truevision TGA (TM) File Format Specification
// Version 2.0


// As Per Targa specs the following are defined:
//	Byte:	8 bits
//	Short:	16 bits
//	Long:	32 bits
//
//	Bytes are stored little_endian.

// IMAGE DESCRIPTOR DEFINES

// COLOUR DEPTHS
// defined for use in field 5.6 image descriptor, #bits for attributes
#define	ATTRIB_16	1
#define	ATTRIB_24	0
#define	ATTRIB_32	8

// IMAGE ORIGIN
#define ORIGIN_L	0
#define	ORIGIN_R	16
#define	ORIGIN_T	32
#define	ORIGIN_B	0

// Image colour type
#define	PSEUDO		1
#define	COL_TRUE	2
#define	MONO		3

// Compression
#define	RLE		8
#define NONE		0

// 16 Bit color positions
#define	BIT16_R	0x7c00
#define	BIT16_G	0x3e0
#define BIT16_B 0x1f

// TGA ERROR CODES
#define TGA_ERR_NONE		1
#define	TGA_ERR_COLOUR		-1
#define	TGA_ERR_COLOR		-1
#define	TGA_ERR_COMP		-2
#define TGA_ERR_ATTRIB		-4
#define TGA_ERR_COLDEPTH	-8
#define	TGA_ERR_UNK			-16

struct RGB {
	unsigned char blue;
	unsigned char green;
	unsigned char red;
};


void	readImage(CFile &ifp, unsigned char *data, int x, int y, int col_depth);

class TGA {
	private:
		// TGA Header Data
		unsigned char	id_length;
		unsigned char	c_map;
		unsigned char	img_type;
		unsigned char	c_map_spec[5];
		unsigned char	image_spec[10];
		// Pointers to data in image_spec;
		short	*x_orig;
		short	*y_orig;
		short	*wid;
		short	*hgt;
		unsigned char	*depth;
		unsigned char	*img_desc;
	
		RGB	*image;
		unsigned char	*data;
		unsigned char	*grey;
		unsigned char	*idf;

	public:
		TGA(short x=0, short y=0, RGB *i=NULL, char comp=NONE);
		~TGA();
		void write(CFile &ofp);
		void compress(void) { img_type!=RLE; };
		void noCompress(void) { img_type&=(!RLE); };
		void setX(int X) { *wid = X; }; 
		void setY(int Y) { *hgt = Y; }; 
		int read(CFile &ifp);
		int read(char *filename);
		void close(void);
		char grey16val(unsigned int *);
		char grey24val(unsigned char *);

		int getX() { return *wid; };
		int getY() { return *hgt; };
		char getComp() { return img_type&RLE; };
		char getColDepth() { return *depth; }; 
		char getIdField() { return id_length; };
		char getColType() { return img_type; };
		char getImgDesc() { return *img_desc; };
		unsigned char *getGreyData() { return grey; };
		int get_width( ) { return *wid; };
		int get_height( ) { return *hgt; };
		int get_grey_value(int gx, int gy) { return grey[gx+gy*(*wid)];} ;
		void	deleteData(void);
};

typedef TGA TGAfile;
