//////////////////////////////////////////////////////////////////////////
// TSXtruder 1.0 by Bendy's 3D Systems
// Copyright (c) 1999
//
// Based on Extruder 1.1 by Bryce Denney
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
//////////////////////////////////////////////////////////////////////////
// TSXDialog.cpp : implementation file
// Defines the dialog box for extruder and calls the
// functions that do the work
//

#include "stdafx.h"
#include "ExtrudeTSX.h"
#include "tga.h"
#include "TSXDialog.h"

#include "hfield.h"
#include "tsx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTSXDialog dialog

// Constructor for dialog
CTSXDialog::CTSXDialog(void)
{
	// Set up initial values for extruder
	//{{AFX_DATA_INIT(CTSXDialog)
	m_xres = 20;
	m_yres = 20;
	m_base = 1.0f;
	m_height = 1.0f;
	m_filename_out = _T("No File");
	m_texture = FALSE;
	m_bump = FALSE;
	m_xsize = 2.0f;
	m_ysize = 2.0f;
	m_smooth = TRUE;
	m_smooth_fac = 0.1;
	m_locksize = TRUE;
	m_lockres = TRUE;
	valid_file = FALSE;
	//}}AFX_DATA_INIT
	m_face_type=F_SEMI;
	max_x=32768;
	max_y=32768;
	res_ratio=size_ratio=1.0f;
	changing_res=FALSE;
	changing_size=FALSE;
	calcFaces(); // Calculate # of faces
}

BOOL
CTSXDialog::Create()
{
	return CDialog::Create(CTSXDialog::IDD);

}

void
CTSXDialog::OnCancel()
{
	// Reset parameters to their default
	m_xres = 20;
	m_yres = 20;

	m_base = 1.0f;
	m_height = 1.0f;

	m_smooth_fac = 0.1;
	m_xsize=2.0f;
	m_ysize=2.0f;
	res_ratio=size_ratio=1.0f;
	max_x=32768;
	max_y=32768;
	
	
	m_filename_out = _T("No File");
	if (valid_file)
		tga.close();
	valid_file = FALSE;
	changing_res=FALSE;
	//}}AFX_DATA_INIT
	calcFaces();
	DestroyWindow();
}

// Data verification, checks that val is positive.
// If not, mess is displayed in a message box and
// the data update is failed
void
DDV_IsPosFloat(CDataExchange* pDX, float val, int mess)
{
	if (val < 0)
	{
		AfxMessageBox(mess);
		pDX->Fail();
	}
}

void CTSXDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTSXDialog)
	DDX_Control(pDX, IDC_PROGRESS1, m_progress_bar);
	DDX_Text(pDX, IDC_X, m_xres);
	DDX_Text(pDX, IDC_Y, m_yres);
	DDX_Text(pDX, IDC_NFACES, m_nfaces);
	DDX_Text(pDX, IDC_BASE, m_base);
	DDX_Text(pDX, IDC_HEIGHT, m_height);
	DDX_Text(pDX, IDC_FILEOUT, m_filename_out);
	DDX_Check(pDX, IDC_TEXTURE, m_texture);
	DDX_Check(pDX, IDC_BUMP, m_bump);
	DDX_Text(pDX, IDC_XSIZE, m_xsize);
	DDX_Text(pDX, IDC_YSIZE, m_ysize);
	DDX_Check(pDX, IDC_SMOOTH, m_smooth);
	DDX_Text(pDX, IDC_SMOOTH_FAC, m_smooth_fac);
	DDX_Check(pDX, IDC_LOCKSIZE, m_locksize);
	DDX_Check(pDX, IDC_LOCKRES, m_lockres);
	//}}AFX_DATA_MAP
	// Check other parameters are positive
	DDV_IsPosFloat(pDX,m_base,IDS_INVALIDBASE);
	DDV_IsPosFloat(pDX,m_height,IDS_INVALIDHEIGHT);
	DDV_IsPosFloat(pDX,(float)m_xres,IDS_INVALIDDIM);
	DDV_IsPosFloat(pDX,(float)m_yres,IDS_INVALIDDIM);
	DDV_IsPosFloat(pDX,(float)m_smooth_fac,IDS_INVALIDSF);
}


BEGIN_MESSAGE_MAP(CTSXDialog, CDialog)
	//{{AFX_MSG_MAP(CTSXDialog)
	ON_BN_CLICKED(IDC_MAKE, OnMake)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_FILENAME, OnGetFile)
	ON_BN_CLICKED(IDC_TRIANGLE, OnRadioTriangle)
	ON_BN_CLICKED(IDC_PLANE, OnRadioPlane)
	ON_BN_CLICKED(IDC_SEMI, OnRadioSemi)
	ON_BN_CLICKED(IDC_LOCKSIZE, OnLocksize)
	ON_BN_CLICKED(IDC_LOCKRES, OnLockres)
	ON_EN_CHANGE(IDC_Y, OnChangeY)
	ON_EN_CHANGE(IDC_X, OnChangeX)
	ON_BN_CLICKED(IDC_XRMINUS, OnXrminus)
	ON_BN_CLICKED(IDC_XRPLUS, OnXrplus)
	ON_BN_CLICKED(IDC_MZSMINUS, OnMzsminus)
	ON_BN_CLICKED(IDC_MZSPLUS, OnMzsplus)
	ON_BN_CLICKED(IDC_PZSMINUS, OnPzsminus)
	ON_BN_CLICKED(IDC_PZSPLUS, OnPzsplus)
	ON_BN_CLICKED(IDC_XSMINUS, OnXsminus)
	ON_BN_CLICKED(IDC_XSPLUS, OnXsplus)
	ON_BN_CLICKED(IDC_YRMINUS, OnYrminus)
	ON_BN_CLICKED(IDC_YRPLUS, OnYrplus)
	ON_BN_CLICKED(IDC_YSMINUS, OnYsminus)
	ON_BN_CLICKED(IDC_YSPLUS, OnYsplus)
	ON_EN_CHANGE(IDC_XSIZE, OnChangeXsize)
	ON_EN_CHANGE(IDC_YSIZE, OnChangeYsize)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CTSXDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	((CButton*) GetDlgItem(IDC_SEMI))->SetCheck(1);
	if (!xr_plus.AutoLoad(IDC_XRPLUS,this))
		return FALSE;
	if (!xr_minus.AutoLoad(IDC_XRMINUS,this))
		return FALSE;
	if (!yr_plus.AutoLoad(IDC_YRPLUS,this))
		return FALSE;
	if (!yr_minus.AutoLoad(IDC_YRMINUS,this))
		return FALSE;
	if (!xs_plus.AutoLoad(IDC_XSPLUS,this))
		return FALSE;
	if (!xs_minus.AutoLoad(IDC_XSMINUS,this))
		return FALSE;
	if (!ys_plus.AutoLoad(IDC_YSPLUS,this))
		return FALSE;
	if (!ys_minus.AutoLoad(IDC_YSMINUS,this))
		return FALSE;

	if (!pz_plus.AutoLoad(IDC_PZSPLUS,this))
		return FALSE;
	if (!pz_minus.AutoLoad(IDC_PZSMINUS,this))
		return FALSE;
	if (!mz_plus.AutoLoad(IDC_MZSPLUS,this))
		return FALSE;
	if (!mz_minus.AutoLoad(IDC_MZSMINUS,this))
		return FALSE;


	if (!extrude.AutoLoad(IDC_MAKE,this))
		return FALSE;
	if (!del.AutoLoad(IDC_DELETE,this))
		return FALSE;
	if (!save.AutoLoad(IDC_FILENAME,this))
		return FALSE;
	//((CButton*) GetDlgItem(IDC_LOCKRES))->SendMessage(BM_SETIMAGE,IMAGE_BITMAP,IDB_LOCK);	
	GetClientRect(&client_rect);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CTSXDialog message handlers

// Opens a dialog box to fetch a TGA file.
// Checks if file is 24 bit grayscale (this is
// not a complete check, some invalid files may
// get through)
// If the file is valid it is read in and the x and
// y resolution is set to the size of the file.
// Number of faces is calculated and make button is
// activated
void CTSXDialog::OnGetFile() 
{
	// TODO: Add your control notification handler code here
	
	double big_size;	
	// Make the dialog box
	CFileDialog	File(TRUE,"tga","*.tga", OFN_HIDEREADONLY |OFN_OVERWRITEPROMPT,"Targa Files (*.tga)|*.tga||");

	// Get the filename
	if (File.DoModal()==IDOK)
	{
			filename = File.GetPathName();
			if (valid_file)		// close previous file
				tga.close();
	}
	else
		return;
	
	int err;
	// Read file in.
	if ((err=tga.read(filename.GetBuffer(128)))!=TGA_ERR_NONE)
	{	// if there is an error set file as invalid
		// and give a message as to why the file
		// couldn't be opened 
		valid_file=FALSE;
		filename = _T("Invalid File");
		switch(err)
		{
		case(TGA_ERR_UNK):	AfxMessageBox(IDS_TGAFILEUNK);break;
//		case(TGAFILEERR):	AfxMessageBox(IDS_TGAFILEERR);break;
		case(TGA_ERR_COLOUR):	AfxMessageBox(IDS_TGANOT24B);break;
		case(TGA_ERR_COLDEPTH):	AfxMessageBox(IDS_TGANOT24B);break;
		case(TGA_ERR_COMP):		AfxMessageBox(IDS_TGACOMP);break;
		case(TGA_ERR_ATTRIB): AfxMessageBox(IDS_TGAATTRIB);break;
		}
		UpdateFileOut(filename);
		((CButton*) GetDlgItem(IDC_MAKE))->EnableWindow(FALSE);
	}
	else
	{
		// else set default parameter values from file
		max_x = m_xres = tga.get_width();
		max_y = m_yres = tga.get_height();
		calcFaces();
	
		// NEW FOR BETA 2
		// calculate object size
		m_xsize=(float)m_xres;
		m_ysize=(float)m_yres;
		
		if (m_xsize > m_ysize)
			big_size = m_xsize;
		else
			big_size = m_ysize;
	

		m_xsize = float(m_xsize / pow(10.0,int(log10(big_size))));
		m_ysize = float(m_ysize / pow(10.0,int(log10(big_size))));
		res_ratio = size_ratio = m_xsize/m_ysize;
		valid_file=TRUE;
		m_smooth=FALSE;
		UpdateData(FALSE);
		UpdateFileOut(File.GetFileName());
		// call enable window on make button
		((CButton*) GetDlgItem(IDC_MAKE))->EnableWindow(TRUE);
		// Disable smoothing controls as we are at max res;
		((CButton*) GetDlgItem(IDC_SMOOTH))->EnableWindow(FALSE);
		((CButton*) GetDlgItem(IDC_SMOOTH_FAC))->EnableWindow(FALSE);
	}
}

// Called when it's time to make the object
void CTSXDialog::OnMake() 
{
	// TODO: Add your control notification handler code here

	CString	str;
	if (!valid_file) // Do we have a file read in?
		return;
	
	if (UpdateData(TRUE))	// if parameters are valid
	{						// then make the object
		calcFaces();	 // How many faces?
		
		Hfield hf(m_xres,m_yres,m_base,m_height,&tga,m_xsize/2,m_ysize/2,m_smooth,m_smooth_fac,m_face_type,filename.GetBuffer(filename.GetLength()),m_texture,m_bump);

		
		SetProgress(0);
		m_nfaces=hf.insertObject(*this);

		SetProgress(0);
		UpdateData(FALSE);
		
		// Activate Delete Button
		((CButton*) GetDlgItem(IDC_DELETE))->EnableWindow(TRUE);
	}

}

// Display Cstring str in the filename box
void
CTSXDialog::UpdateFileOut(CString str)
{
	UpdateData(TRUE);
	m_filename_out=str;
	UpdateData(FALSE);
}

// Delete the object and deactivate the delete button
void CTSXDialog::OnDelete() 
{

	// TODO: Add your control notification handler code here
	tsxSobjDelete(tsxGetCurrentSelection());

	((CButton*)GetDlgItem(IDC_DELETE))->EnableWindow(FALSE);
}


void
CTSXDialog::checkSmoothing(void)
{
	if (m_xres > max_x/2 || m_yres > max_y/2)
	{
		((CButton*) GetDlgItem(IDC_SMOOTH))->EnableWindow(FALSE);
		((CButton*) GetDlgItem(IDC_SMOOTH_FAC))->EnableWindow(FALSE);
		m_smooth=false;
	}
	else
	{
		((CButton*) GetDlgItem(IDC_SMOOTH))->EnableWindow(TRUE);
		((CButton*) GetDlgItem(IDC_SMOOTH_FAC))->EnableWindow(TRUE);
	}
}

void CTSXDialog::OnRadioTriangle() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	m_face_type=F_TRIANGLE;
	calcFaces();
	UpdateData(FALSE);
}

void CTSXDialog::OnRadioPlane() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	m_face_type=F_PLANE;
	calcFaces();
	UpdateData(FALSE);
}

void CTSXDialog::OnRadioSemi() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	m_face_type=F_SEMI;
	calcFaces();
	UpdateData(FALSE);
}
void
CTSXDialog::calcFaces(void)
{
	switch(m_face_type)
	{
		case(F_PLANE):m_nfaces = 5+(m_xres-1)*(m_yres-1);break;
		case(F_TRIANGLE):m_nfaces = 5+2*(m_xres-1)*(m_yres-1);break;
		case(F_SEMI):m_nfaces = 5+2*(m_xres-1)*(m_yres-1);break;
	}
}

void CTSXDialog::OnChangeX() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_CHANGE flag ORed into the lParam mask.
	
	// TODO: Add your control notification handler code here
	if (!changing_res)
	{
		changing_res=TRUE;
		UpdateData(TRUE);
		if (m_xres>max_x)
			m_xres=max_x;
		if (m_lockres)
			m_yres=unsigned int(m_xres/res_ratio);
		calcFaces();
		checkSmoothing();
		UpdateData(FALSE);
		changing_res=FALSE;
	}
}

void CTSXDialog::OnChangeY()
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_CHANGE flag ORed into the lParam mask.
	
	// TODO: Add your control notification handler code here
	
	if (!changing_res)
	{
		changing_res=TRUE;
		UpdateData(TRUE);
		if (m_yres>max_y)
			m_yres=max_y;

		if (m_lockres)
			m_xres=unsigned int(m_yres*res_ratio);
		calcFaces();
		checkSmoothing();

		UpdateData(FALSE);
		changing_res=FALSE;
	}
}

void CTSXDialog::OnLocksize() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	if (m_locksize)
		size_ratio=m_xsize/m_ysize;
}

void CTSXDialog::OnLockres() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	if (m_lockres)
		res_ratio=m_xres/float(m_yres);
}


void CTSXDialog::SetProgress(int val)
{
	m_progress_bar.SetPos(val);
}

void CTSXDialog::OnXrplus() 
{
	// TODO: Add your control notification handler code here
	m_xres++;
	UpdateData(FALSE);
	OnChangeX();
}

void CTSXDialog::OnXrminus() 
{
	// TODO: Add your control notification handler code here
	m_xres--;
	UpdateData(FALSE);
	OnChangeX();
}

void CTSXDialog::OnMzsminus() 
{
	// TODO: Add your control notification handler code here
	m_base-=(float)0.1;
	UpdateData(FALSE);
}

void CTSXDialog::OnMzsplus() 
{
	// TODO: Add your control notification handler code here
	m_base+=(float)0.1;
	UpdateData(FALSE);
	
}

void CTSXDialog::OnPzsminus() 
{
	// TODO: Add your control notification handler code here
	m_height-=(float)0.1;
	UpdateData(FALSE);
}

void CTSXDialog::OnPzsplus() 
{
	// TODO: Add your control notification handler code here
	m_height+=(float)0.1;
	UpdateData(FALSE);
	
}

void CTSXDialog::OnXsminus() 
{
	// TODO: Add your control notification handler code here
	m_xsize-=(float)0.1;
	UpdateData(FALSE);
	OnChangeXsize();
}

void CTSXDialog::OnXsplus() 
{
	// TODO: Add your control notification handler code here
	m_xsize+=(float)0.1;
	UpdateData(FALSE);
	OnChangeXsize();
}

void CTSXDialog::OnYrminus() 
{
	// TODO: Add your control notification handler code here
	m_yres--;
	UpdateData(FALSE);
	OnChangeY();
	
}

void CTSXDialog::OnYrplus() 
{
	// TODO: Add your control notification handler code here
	m_yres++;
	UpdateData(FALSE);
	OnChangeY();

}

void CTSXDialog::OnYsminus() 
{
	// TODO: Add your control notification handler code here
	m_ysize-=(float)0.1;
	UpdateData(FALSE);
	OnChangeYsize();
	
}

void CTSXDialog::OnYsplus() 
{
	// TODO: Add your control notification handler code here
	m_ysize+=(float)0.1;
	UpdateData(FALSE);
	OnChangeYsize();
}

void CTSXDialog::OnChangeXsize() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_CHANGE flag ORed into the lParam mask.
	
	// TODO: Add your control notification handler code here
	
	char	line[8];
	int		num;

	num=(int)(((CEdit*) GetDlgItem(IDC_XSIZE))->SendMessage(WM_GETTEXT,8,(long)line));
	if (!changing_size && m_locksize && line[num-1]!='.')
	{
		changing_size=TRUE;
		UpdateData(TRUE);
		if (m_locksize)
			m_ysize=m_xsize/size_ratio;
		UpdateData(FALSE);
		changing_size=FALSE;
	}

}

void CTSXDialog::OnChangeYsize() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_CHANGE flag ORed into the lParam mask.
	
	// TODO: Add your control notification handler code here

	char	line[8];
	int		num;

	num=(int)(((CEdit*) GetDlgItem(IDC_YSIZE))->SendMessage(WM_GETTEXT,8,(long)line));
	if (!changing_size && m_locksize && line[num-1]!='.')
	{
		changing_size=TRUE;
		UpdateData(TRUE);
		if (m_locksize)
			m_xsize=m_ysize*size_ratio;
		UpdateData(FALSE);
		changing_size=FALSE;
	}	
}



void CTSXDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CDC dcMem;
	CBitmap	m_image;

	m_image.LoadBitmap(IDB_DIALOGBG);
			
	dcMem.CreateCompatibleDC(&dc);

	CBitmap* pOldBitmap = dcMem.SelectObject(&m_image);
	dc.BitBlt(0,0,client_rect.right,client_rect.bottom, &dcMem,0,0,SRCCOPY);
	dcMem.SelectObject(pOldBitmap);

	// Do not call CDialog::OnPaint() for painting messages
}

